@@ -61,18 +61,16 @@ def qdump__QByteArray(d, value):
         d.putValue(p, "latin1", elided=elided)
     elif displayFormat == SeparateLatin1StringFormat:
         d.putValue(p, "latin1", elided=elided)
-        d.putField("editformat", DisplayLatin1String)
-        d.putField("editvalue", d.encodeByteArray(value, limit=100000))
+        d.putDisplay("latin1:separate", d.encodeByteArray(value, limit=100000))
     elif displayFormat == Utf8StringFormat:
         d.putValue(p, "utf8", elided=elided)
     elif displayFormat == SeparateUtf8StringFormat:
         d.putValue(p, "utf8", elided=elided)
-        d.putField("editformat", DisplayUtf8String)
-        d.putField("editvalue", d.encodeByteArray(value, limit=100000))
+        d.putDisplay("utf8:separate", d.encodeByteArray(value, limit=100000))
     if d.isExpanded():
         d.putArrayData(data, size, d.charType())
 
-def qdump__QByteArrayData(d, value):
+def qdump__QArrayData(d, value):
     data, size, alloc = d.byteArrayDataHelper(d.addressOf(value))
     d.check(alloc == 0 or (0 <= size and size <= alloc and alloc <= 100000000))
     d.putValue(d.readMemory(data, size), "latin1")
@@ -82,6 +80,9 @@ def qdump__QByteArrayData(d, value):
             d.putIntItem("size", size)
             d.putIntItem("alloc", alloc)
 
+def qdump__QByteArrayData(d, value):
+    qdump__QArrayData(d, value)
+
 
 def qdump__QBitArray(d, value):
     data, basize, alloc = d.byteArrayDataHelper(d.extractPointer(value["d"]))
@@ -897,13 +898,12 @@ def qdump__QImage(d, value):
         #   filename = file[1].replace("\\", "\\\\")
         #   gdb.execute("dump binary memory %s %s %s" %
         #       (filename, bits, bits + nbytes))
-        #   d.putDisplay(DisplayImageFile, " %d %d %d %d %s"
+        #   d.putDisplay('imagefile:separate', " %d %d %d %d %s"
         #       % (width, height, nbytes, iformat, filename))
-        d.putField("editformat", DisplayImageData)
-        d.put('editvalue="')
-        d.put('%08x%08x%08x%08x' % (width, height, nbytes, iformat))
-        d.put(d.readMemory(bits, nbytes))
-        d.put('",')
+        d.putDisplay('imagedata:separate',
+                     '%08x%08x%08x%08x' % (width, height, nbytes, iformat)
+                        + d.readMemory(bits, nbytes))
+
 
 
 def qdump__QLinkedList(d, value):
@@ -1089,431 +1089,26 @@ def qdump__QVariantMap(d, value):
     qdump__QMap(d, value)
 
 
-def qdump__QMetaObjectPrivate(d, value):
-    d.putEmptyValue()
-    d.putNumChild(1)
-    if d.isExpanded():
-        with Children(d):
-            # int revision;
-            # int className;
-            # int classInfoCount, classInfoData;
-            # int methodCount, methodData;
-            # int propertyCount, propertyData;
-            # int enumeratorCount, enumeratorData;
-            # int constructorCount, constructorData; //since revision 2
-            # int flags; //since revision 3
-            # int signalCount; //since revision 4
-            d.putIntItem("revision", value["revision"])
-            d.putIntItem("methodCount", value["methodCount"])
-            d.putIntItem("propertyCount", value["propertyCount"])
-            d.putIntItem("enumeratorCount", value["enumeratorCount"])
-            d.putIntItem("constructorCount", value["constructorCount"])
-            d.putIntItem("flags", value["flags"])
-            d.putIntItem("signalCount", value["signalCount"])
+def qdump__QMetaMethod(d, value):
+    d.putQMetaStuff(value, "QMetaMethod")
 
+def qdump__QMetaEnum(d, value):
+    d.putQMetaStuff(value, "QMetaEnum")
 
+def qdump__QMetaProperty(d, value):
+    d.putQMetaStuff(value, "QMetaProperty")
+
+def qdump__QMetaClassInfo(d, value):
+    d.putQMetaStuff(value, "QMetaClassInfo")
 
 def qdump__QMetaObject(d, value):
     d.putEmptyValue()
     d.putNumChild(1)
     if d.isExpanded():
         with Children(d):
-            dd = value["d"]
-            d.putSubItem("d", dd)
-            data = d.extractPointer(dd["data"])
+            d.putQObjectGutsHelper(0, 0, -1, d.addressOf(value), "QMetaObject")
+            d.putMembersItem(value)
 
-            propertyNames = d.staticQObjectPropertyNames(value)
-            propertyIndex = 0
-            for propertyName in propertyNames:
-                with SubItem(d, "property_%s" % propertyIndex):
-                    d.putValue(propertyName)
-                    propertyIndex += 1
-
-           #byteArrayDataType = d.lookupType(d.qtNamespace() + "QByteArrayData")
-           #byteArrayDataSize = byteArrayDataType.sizeof
-           #sd = d.extractPointer(dd["stringdata"])
-           #stringdata, size, alloc = d.byteArrayDataHelper(sd)
-
-           #propertyCount = d.extractInt(data + 24)
-           #propertyData = d.extractInt(data + 28)
-
-           ## This is the 'data' member in the qt_meta_stringdata_qobject__*_t struct
-           #d.putIntItem("_byteArrayDataSize", byteArrayDataSize)
-           #d.putAddressItem("_data", data)
-           #d.putAddressItem("_sd_", stringdata)
-           #with SubItem(d, "_sd"):
-           #    d.putValue(d.readMemory(stringdata, size), "latin1")
-           #with SubItem(d, "_cn"):
-           #    d.putValue(d.readMemory(stringdata + d.extractInt(data + 4), size), "latin1")
-
-           #for i in range(propertyCount):
-           #    with SubItem(d, "property_%s" % i):
-           #        x = data + (propertyData + 3 * i) * 4
-           #        literal = sd + d.extractInt(x) * byteArrayDataSize
-           #        ldata, lsize, lalloc = d.byteArrayDataHelper(literal)
-           #        d.putValue(d.readMemory(ldata, lsize), "latin1")
-
-           #        d.putNumChild(1)
-           #        if d.isExpanded():
-           #            with Children(d):
-           #                if d.isExpanded():
-           #                    d.putAddressItem("_literal", literal)
-           #                    d.putIntItem("__data", ldata)
-           #                    d.putIntItem("__size", lsize)
-           #                    d.putIntItem("__alloc", lalloc)
-           #                    d.putIntItem("name", d.extractInt(x))
-           #                    d.putIntItem("type", d.extractInt(x + 4))
-           #                    d.putIntItem("flags", d.extractInt(x + 8))
-
-            methodCount = d.extractInt(data + 16)
-            methodData = d.extractInt(data + 20)
-            for i in range(methodCount):
-                with SubItem(d, "method_%s" % i):
-                    x = data + (methodData + 5 * i) * 4
-                    #d.putEmptyValue()
-                    d.putValue(d.readCString(stringdata + d.extractInt(x)))
-                    d.putNumChild(1)
-                    if d.isExpanded():
-                        with Children(d):
-                            if d.isExpanded():
-                                d.putIntItem("name", d.extractInt(x))
-                                d.putIntItem("argc", d.extractInt(x + 4))
-                                d.putIntItem("argv", d.extractInt(x + 8))
-                                d.putIntItem("type", d.extractInt(x + 12))
-                                d.putIntItem("flags", d.extractInt(x + 16))
-
-            d.putSubItem("stringData", dd["stringdata"])
-            d.putIntItem("revision", d.extractInt(data))
-            d.putIntItem("className", d.extractInt(data + 4))
-            d.putIntItem("classInfoCount", d.extractInt(data + 8))
-            d.putIntItem("className", d.extractInt(data + 12))
-            d.putIntItem("methodCount", d.extractInt(data + 16))
-            d.putIntItem("methodData", d.extractInt(data + 20))
-            d.putIntItem("propertyCount", d.extractInt(data + 24))
-            d.putIntItem("propertyData", d.extractInt(data + 28))
-            d.putIntItem("enumeratorCount", d.extractInt(data + 32))
-            d.putIntItem("enumeratorData", d.extractInt(data + 36))
-            d.putIntItem("constructorCount", d.extractInt(data + 40))
-            d.putIntItem("constructorData", d.extractInt(data + 44))
-            d.putIntItem("flags", d.extractInt(data + 48))
-            d.putIntItem("signalCount", d.extractInt(data + 52))
-
-def _qdump__QObject(d, value):
-    d.putQObjectNameValue(value)
-    ns = d.qtNamespace()
-
-    try:
-        privateTypeName = ns + "QObjectPrivate"
-        privateType = d.lookupType(privateTypeName)
-        staticMetaObject = value["staticMetaObject"]
-    except:
-        d.putPlainChildren(value)
-        return
-    #warn("SMO: %s " % staticMetaObject)
-    #warn("SMO DATA: %s " % staticMetaObject["d"]["stringdata"])
-    superData = staticMetaObject["d"]["superdata"]
-    #warn("SUPERDATA: %s" % superData)
-    #while not d.isNull(superData):
-    #    superData = superData.dereference()["d"]["superdata"]
-    #    warn("SUPERDATA: %s" % superData)
-
-    if privateType is None:
-        #d.putValue(d.cleanAddress(d.pointerValue(value))
-        d.putPlainChildren(value)
-        return
-
-    #warn("OBJECTNAME: %s " % objectName)
-    dd = value["d_ptr"]["d"]
-    d_ptr = dd.cast(privateType.pointer()).dereference()
-    #warn("D_PTR: %s " % d_ptr)
-    mo = d_ptr["metaObject"]
-    if d.isNull(mo):
-        mo = staticMetaObject
-    #warn("MO: %s " % mo)
-    #warn("MO.D: %s " % mo["d"])
-    metaData = mo["d"]["data"]
-    metaStringData = mo["d"]["stringdata"]
-    # This is char * in Qt 4 and ByteArrayData * in Qt 5.
-    # Force it to the char * data in the Qt 5 case.
-    try:
-        offset = metaStringData["offset"]
-        metaStringData = metaStringData.cast(d.charPtrType()) + int(offset)
-    except:
-        pass
-
-    #extradata = mo["d"]["extradata"]   # Capitalization!
-    #warn("METADATA: %s " % metaData)
-    #warn("STRINGDATA: %s " % metaStringData)
-    #warn("TYPE: %s " % value.type)
-    #warn("INAME: %s " % d.currentIName)
-    d.putEmptyValue()
-    #QSignalMapper::staticMetaObject
-    #d.checkRef(d_ptr["ref"])
-    d.putNumChild(4)
-    if d.isExpanded():
-      with Children(d):
-        d.putQObjectGuts(value)
-
-        # Local data.
-        if privateTypeName != ns + "QObjectPrivate":
-            if not privateType is None:
-              with SubItem(d, "data"):
-                d.putEmptyValue()
-                d.putNoType()
-                d.putPlainChildren(d_ptr, False)
-
-        d.putFields(value)
-        # Parent and children.
-        if d.stripClassTag(str(value.type)) == ns + "QObject":
-            d.putSubItem("parent", d_ptr["parent"])
-            d.putSubItem("children", d_ptr["children"])
-
-        # Metaobject.
-        d.putSubItem("metaobject", mo)
-
-        # Dynamic Properties.
-        with SubItem(d, "dynamics"):
-            # Prolog
-            extraData = d_ptr["extraData"]   # Capitalization!
-            if d.isNull(extraData):
-                dynamicPropertyCount = 0
-            else:
-                extraDataType = d.lookupType(
-                    ns + "QObjectPrivate::ExtraData").pointer()
-                extraData = extraData.cast(extraDataType)
-                ed = extraData.dereference()
-                names = ed["propertyNames"]
-                values = ed["propertyValues"]
-                #userData = ed["userData"]
-                namesBegin = names["d"]["begin"]
-                namesEnd = names["d"]["end"]
-                namesArray = names["d"]["array"]
-                dynamicPropertyCount = namesEnd - namesBegin
-
-            d.putNoType()
-            d.putItemCount(dynamicPropertyCount)
-
-            if d.isExpanded() and d.isGdb:
-                import gdb
-                # FIXME: Make this global. Don't leak.
-                variant = "'%sQVariant'" % ns
-                # Avoid malloc symbol clash with QVector
-                gdb.execute("set $d = (%s*)calloc(sizeof(%s), 1)"
-                    % (variant, variant))
-                gdb.execute("set $d.d.is_shared = 0")
-
-                with Children(d):
-                    dummyType = d.voidPtrType().pointer()
-                    namesType = d.lookupType(ns + "QByteArray")
-                    valuesBegin = values["d"]["begin"]
-                    valuesEnd = values["d"]["end"]
-                    valuesArray = values["d"]["array"]
-                    valuesType = d.lookupType(ns + "QVariant")
-                    p = namesArray.cast(dummyType) + namesBegin
-                    q = valuesArray.cast(dummyType) + valuesBegin
-                    for i in xrange(dynamicPropertyCount):
-                        with SubItem(d, i):
-                            pp = p.cast(namesType.pointer()).dereference();
-                            d.putField("key", d.encodeByteArray(pp))
-                            d.putField("keyencoded", "latin1")
-                            qq = q.cast(valuesType.pointer().pointer())
-                            qq = qq.dereference();
-                            d.putField("address", d.cleanAddress(qq))
-                            d.putField("exp", "*(%s*)%s"
-                                 % (variant, d.cleanAddress(qq)))
-                            t = qdump__QVariant(d, qq)
-                            # Override the "QVariant (foo)" output.
-                            d.putBetterType(t)
-                        p += 1
-                        q += 1
-
-
-        # Connections.
-        with SubItem(d, "connections"):
-            d.putNoType()
-            connections = d_ptr["connectionLists"]
-            connectionListCount = 0
-            if not d.isNull(connections):
-                connectionListCount = connections["d"]["size"]
-            d.putItemCount(connectionListCount, 0)
-            if d.isExpanded():
-                pp = 0
-                with Children(d):
-                    vectorType = d.fieldAt(connections.type.target(), 0).type
-                    innerType = d.templateArgument(vectorType, 0)
-                    # Should check:  innerType == ns::QObjectPrivate::ConnectionList
-                    p = gdb.Value(connections["p"]["array"]).cast(innerType.pointer())
-                    for i in xrange(connectionListCount):
-                        first = p.dereference()["first"]
-                        while not d.isNull(first):
-                            with SubItem(d, pp):
-                                connection = first.dereference()
-                                d.putItem(connection)
-                                d.putValue(connection["callFunction"])
-                            first = first["nextConnectionList"]
-                            # We need to enforce some upper limit.
-                            pp += 1
-                            if pp > 1000:
-                                break
-                        p += 1
-                if pp < 1000:
-                    d.putItemCount(pp)
-
-
-        # Active connection.
-        with SubItem(d, "currentSender"):
-            d.putNoType()
-            sender = d_ptr["currentSender"]
-            d.putPointerValue(sender)
-            if d.isNull(sender):
-                d.putNumChild(0)
-            else:
-                d.putNumChild(1)
-                if d.isExpanded():
-                    with Children(d):
-                        # Sending object
-                        d.putSubItem("object", sender["sender"])
-                        # Signal in sending object
-                        with SubItem(d, "signal"):
-                            d.putValue(sender["signal"])
-                            d.putNoType()
-                            d.putNumChild(0)
-
-# QObject
-
-#   static const uint qt_meta_data_QObject[] = {
-
-#   int revision;
-#   int className;
-#   int classInfoCount, classInfoData;
-#   int methodCount, methodData;
-#   int propertyCount, propertyData;
-#   int enumeratorCount, enumeratorData;
-#   int constructorCount, constructorData; //since revision 2
-#   int flags; //since revision 3
-#   int signalCount; //since revision 4
-
-#    // content:
-#          4,       // revision
-#          0,       // classname
-#          0,    0, // classinfo
-#          4,   14, // methods
-#          1,   34, // properties
-#          0,    0, // enums/sets
-#          2,   37, // constructors
-#          0,       // flags
-#          2,       // signalCount
-
-#  /* 14 */
-
-#    // signals: signature, parameters, type, tag, flags
-#          9,    8,    8,    8, 0x05,
-#         29,    8,    8,    8, 0x25,
-
-#  /* 24 */
-#    // slots: signature, parameters, type, tag, flags
-#         41,    8,    8,    8, 0x0a,
-#         55,    8,    8,    8, 0x08,
-
-#  /* 34 */
-#    // properties: name, type, flags
-#         90,   82, 0x0a095103,
-
-#  /* 37 */
-#    // constructors: signature, parameters, type, tag, flags
-#        108,  101,    8,    8, 0x0e,
-#        126,    8,    8,    8, 0x2e,
-
-#          0        // eod
-#   };
-
-#   static const char qt_meta_stringdata_QObject[] = {
-#       "QObject\0\0destroyed(QObject*)\0destroyed()\0"
-#       "deleteLater()\0_q_reregisterTimers(void*)\0"
-#       "QString\0objectName\0parent\0QObject(QObject*)\0"
-#       "QObject()\0"
-#   };
-
-
-# QSignalMapper
-
-#   static const uint qt_meta_data_QSignalMapper[] = {
-
-#    // content:
-#          4,       // revision
-#          0,       // classname
-#          0,    0, // classinfo
-#          7,   14, // methods
-#          0,    0, // properties
-#          0,    0, // enums/sets
-#          0,    0, // constructors
-#          0,       // flags
-#          4,       // signalCount
-
-#    // signals: signature, parameters, type, tag, flags
-#         15,   14,   14,   14, 0x05,
-#         27,   14,   14,   14, 0x05,
-#         43,   14,   14,   14, 0x05,
-#         60,   14,   14,   14, 0x05,
-
-#    // slots: signature, parameters, type, tag, flags
-#         77,   14,   14,   14, 0x0a,
-#         90,   83,   14,   14, 0x0a,
-#        104,   14,   14,   14, 0x08,
-
-#          0        // eod
-#   };
-
-#   static const char qt_meta_stringdata_QSignalMapper[] = {
-#       "QSignalMapper\0\0mapped(int)\0mapped(QString)\0"
-#       "mapped(QWidget*)\0mapped(QObject*)\0"
-#       "map()\0sender\0map(QObject*)\0"
-#       "_q_senderDestroyed()\0"
-#   };
-
-#   const QMetaObject QSignalMapper::staticMetaObject = {
-#       { &QObject::staticMetaObject, qt_meta_stringdata_QSignalMapper,
-#         qt_meta_data_QSignalMapper, 0 }
-#   };
-
-
-
-# // Meta enumeration helpers
-# static inline void dumpMetaEnumType(QDumper &d, const QMetaEnum &me)
-# {
-#     QByteArray type = me.scope()
-#     if !type.isEmpty())
-#         type += "::"
-#     type += me.name()
-#     d.putField("type", type.constData())
-# }
-#
-# static inline void dumpMetaEnumValue(QDumper &d, const QMetaProperty &mop,
-#                                      int value)
-# {
-#
-#     const QMetaEnum me = mop.enumerator()
-#     dumpMetaEnumType(d, me)
-#     if const char *enumValue = me.valueToKey(value)) {
-#         d.putValue(enumValue)
-#     } else {
-#         d.putValue(value)
-#     }
-#     d.putField("numchild", 0)
-# }
-#
-# static inline void dumpMetaFlagValue(QDumper &d, const QMetaProperty &mop,
-#                                      int value)
-# {
-#     const QMetaEnum me = mop.enumerator()
-#     dumpMetaEnumType(d, me)
-#     const QByteArray flagsValue = me.valueToKeys(value)
-#     if flagsValue.isEmpty():
-#         d.putValue(value)
-#     else:
-#         d.putValue(flagsValue.constData())
-#     d.putNumChild(0)
-# }
 
 def qdump__QPixmap(d, value):
     offset = (3 if d.qtVersion() >= 0x050000 else 2) * d.ptrSize()
@@ -1788,8 +1383,7 @@ def qdump__QString(d, value):
     d.putNumChild(size)
     displayFormat = d.currentItemFormat()
     if displayFormat == SeparateFormat:
-        d.putField("editformat", DisplayUtf16String)
-        d.putField("editvalue", d.encodeString(value, limit=100000))
+        d.putDisplay("utf16:separate", d.encodeString(value, limit=100000))
     if d.isExpanded():
         d.putArrayData(data, size, d.lookupType(d.qtNamespace() + "QChar"))
 
@@ -1940,8 +1534,7 @@ def qdump__QUrl(d, value):
 
         displayFormat = d.currentItemFormat()
         if displayFormat == SeparateFormat:
-            d.putField("editformat", DisplayUtf16String)
-            d.putField("editvalue", url)
+            d.putDisplay("utf16:separate", url)
 
         d.putNumChild(8)
         if d.isExpanded():
@@ -2642,7 +2235,10 @@ def qdumpHelper__QJsonValue(d, data, base, pv):
     if t == 2:
         d.putType("QJsonValue (Number)")
         if latinOrIntValue:
-            d.putValue(v)
+            w = toInteger(v)
+            if w >= 0x4000000:
+                w -= 0x8000000
+            d.putValue(w)
         else:
             data = base + v;
             d.putValue(d.extractBlob(data, 8).extractDouble())
@@ -2743,10 +2339,12 @@ def qdump__QJsonValue(d, value):
         d.putType("QJsonValue (Bool)")
         v = toInteger(value["b"])
         d.putValue("true" if v else "false")
+        d.putNumChild(0)
         return
     if t == 2:
         d.putType("QJsonValue (Number)")
         d.putValue(value["dbl"])
+        d.putNumChild(0)
         return
     if t == 3:
         d.putType("QJsonValue (String)")
